package PureMessage::MessageLog;

use strict;
use base 'Exporter';
our @EXPORT_OK = qw(
    message_logs 
    message_log 
    decode_value 
    encode_value
    split_line
    join_line);

sub message_logs {
    my $milter = shift || "";
    require PureMessage::Config;
    my %logs;

    if ($milter) {
        my $mlog = $PureMessage::Config::config{milter}{$milter}{message_log} || undef;
        push(@{$logs{$mlog}}, $milter) if $mlog;
    }
    else {
    for (keys %{$PureMessage::Config::config{milter}}) {
	my $mlog = $PureMessage::Config::config{milter}{$_}{message_log} || next;
	push(@{$logs{$mlog}}, $_);
    }
    }
    \%logs
}

sub message_log {
    my $prefix = shift;
    my $milter = shift || "";
    if ($prefix) {
	$prefix .= ": ";
    }
    else {
	$prefix = "";
    }
    require PureMessage::Config;
    my $logs = message_logs($milter);
    die $prefix . "No message_log found in $PureMessage::Config::config{pmx_conf}\n" unless %$logs;
    die $prefix . "Multiple logs in $PureMessage::Config::config{pmx_conf}\n" if keys %$logs > 1;
    my($log) = keys %$logs;
    return $log;
}

sub _decode_value {
    # decode a message log value according to the message log spec (a
    # modified form of RFC 2047 q-encoding).
    for ($_[0]) {
	if (s/^\?q\?//) {
	    s/_/ /g;
	    s/=([\da-fA-F]{2})/chr(hex($1))/ge;
	}
    }
}

sub encode_value {
    my $v = shift;
    return $v unless $v;
    if ($v =~ /^\?/ || $v =~ /[^\x21-\x3c\x3e-\x7e]/) {
	
	# encode all non-printing characters, '_' and '='.
	$v =~ s/([^\x20-\x3c\x3e-\x5e\x60-\x7e])/sprintf("=%02X",ord($1))/ge;
	$v =~ s/ /_/g;
	$v = "?q?$v";
    }
    return $v;
}

sub decode_value {
    my $v = shift;
    _decode_value($v) if $v;
    return $v;
}

sub split_line {
    my($ts, @fields) = split(' ', shift);
    return ($ts, map {
	my($k, $v) = split(/=/, $_, 2);
	_decode_value($v) if $v && $v =~ /^\?/;
	($k, $v)
    } @fields);
}

sub join_line {
    my $line = shift; # grab the time stamp if it's there
    while (my ($k, $v) = @_) {
	shift; shift;
	if ($line) {
	    $line .= " $k";
	}
	else { # must be the first element
	    $line = $k;
	}
	if (defined $v and length $v) {
	    $line .= "=" . encode_value($v);
	}
    }
    return $line;
}



1;

=head1 NAME

PureMessage::MessageLog - Parse the message log

=head1 SYNOPSIS

 use PureMessage::MessageLog qw(split_line decode_value join_line);

 while (<LOG>) {
     my($ts, @kv) = split_line($_);
     # ...
 }

or

 while (<LOG>) {
     my($ts, @kv) = split(' ', $line);
     for (@kv) {
         my($k, $v) = split('=', $_, 2);
         # ...
         $v = decode_value($v);
         # ...
     }
 }

or

 print LOG join_line($time_stamp, @data), "\n";
 

=head1 DESCRIPTION

The message log is written by the milter if the C<message_log>
configuration option is given an value.  The file produced contains a
single line per message processed.  A log line contains a timestamp
and some key/value pairs.  The C<PureMessage::MessageLog> module
provides the following functions to parse and emit message log data:

=over

=item $href = message_logs()

Returns a reference to a hash.  The keys of the hash are the
message_logs files used by the currently configured milters.  Each
corresponding value is a reference to an array of milter names.

=item $log = message_log()

=item $log = message_log( $progname )

Returns the message_log file if there is exactly one configured.
Will die otherwise.  If a $progname argument is given, then the
die-message will be prefixed with it.

=item ($ts, @kv) = split_line( $line )

This parses a line from the message log into a timestamp and then a
sequence of key/value pairs.  For fields without a value, the value is
returned as C<undef>.  It might be tempting to assign the @kv to a
hash, but remember that keys might repeat.

=item $v = decode_value( $v )

This decodes a single value.  It is useful in cases where you decide
to do the line splitting yourself for performance reasons.
See the synopsis section above.

=item $line = join_line( $ts, @kv )

Generates a valid log line from a time stamp and an array of key/value
pairs. Keys that don't have values should be followed by an undef. 
Set C<$ts> to undef if you don't want the line prefixed with a time 
stamp.

=item $v = encode_value( $v )

Encodes a single value.

=back

=head1 MESSAGE LOG FILE FORMAT

Each line of the message log consists of space-separated fields.  The
first field of every line is always an ISO timestamp value.  The rest of
the fields are key/value pairs separated with an '='.  Some fields
may consist only of keys, rather than key / value pairs, in which case 
the '=' symbol will not be present. For example:

  2003-01-01T12:00:00 k1=v1 k2=v2 k2=v3 k3=?q?foo_"bar" k4

The keys ('k1', 'k2', 'k3' and 'k4' in the example) are plain strings
following standard identifier syntax:

  a-z\w*

The values can be arbitrary strings.  If the values are simple (do not
contain whitespace or non-printable characters, and do not start with 
'?'), then they are written directly as-is.  Otherwise they are 
quoted using the MIME q-encoding (slightly modified).

The q-encoding is specified in RFC 2047.  Our modification is that we
don't encode a character set and that we don't encode the '?' itself.
A string like "Foo=Bar and Baz" is encoded as "?q?Foo=3DBar_and_Baz".

The q-encoding ensures that it is safe to decode the message log using
a simple Perl loop like shown in the synopsis above.

The following keys are used:

=over

=item a

The milter status action for this message, and the callback that provided the 
final action.

=item b

The result of a blocklist lookup, ok or reject.

=item crpt

The custom report identifier and associated policy specified value.

=item f

The envelope-from address.

=item fur

The first untrusted relay as determined by the trusted-relays file

=item h

The spam rule name. Will repeat if multiple spam rules are hit. 

=item p

The spam probability value for the message; a number between 0 and 1.

=item q

The sendmail queue id of the message.

=item r

The hostname of the relay SMTP server.

=item t

The envelope-to address.  Repeats when there are multiple recipients of the 
message.

=item tm

The time (in seconds) that it took to process the message.

=item v

The virus ID. Will repeat if multiple viruses are found.

=item vs

The message was scanned by the virus engine.

=back

Additional custom keys and key / value pairs can be written to the message 
log using the C<pmx_mark> and C<pmx_mark1> policy actions. The default 
version of the policy.siv adds an C<i> key to indicate messages that 
originated from an internal host, and a C<Size> key with the value of the message 
size, in bytes. The C<pmx_test_mark> policy action adds a C<Test-ID> entry for the 
purpose of tracking the test message from the C<pmx-test> program.

=head1 SEE ALSO

L<pmx-mlog>, L<pmx-policy>, L<PerlMx>
