package PureMessage::MessageLog::System;
use strict;
use PureMessage::Config qw(%config);
use PureMessage::Version qw();
use PureMessage::Service qw();
use ActiveState::Install qw();
use PureMessage::Resources::Location qw();
use Digest::MD5 qw(md5_hex);

use base qw(Exporter);
our @EXPORT_OK = qw(
    add_product_specific_headers
    blocklist_log_location
    blocker_status
    batv_status
    exclude_from_log
    first_untrusted_relay
    interesting_package_information
    license_type
    message_log_location
    state_file_location
    system_id
    system_name
    system_version
    upload_destination
);

sub system_name {
    return "PureMessage for Unix";
}

sub system_version {
    return PureMessage::Version::Version();
}

{
    my $blocker_status = 'not installed';
    my @services = PureMessage::Service->Available;

    for my $service (@services) {
	if ($service->type eq 'blockerd') {
	    if ($service->service_status()) {
		$blocker_status = 'enabled';

                if (my $h = $config{'blocklist'}{'Policy'}) {
                    if ($h->{'block_dynamic'}) {
                        $blocker_status .= '_dynamic';
                    }

                    if ($h->{'block_helo'}) {
                        $blocker_status .= '_helo';
                    }
                }
	    }
	    else {
		$blocker_status = 'disabled';
	    }
	}
    }

    sub blocker_status {
        return $blocker_status;
    }

}

{
    my $policy = policy_siv_cache_location() ;
    sub batv_status {
        return 'unknown' unless -f $policy;

        open my $fh, $policy or return 'unknown';
        for(<$fh>) {
            if(/pmx_batv_watermark/) {
                close $fh;
                return 'enabled';
            }
        }

        close $fh;
        return 'disabled';
    }
}


{
    my $system_id;
    my $serial_no = "lookup failed";
    my $license_type = "x";

    {
        my $location_id = "lookup failed";
        my $location_ip = "lookup failed";

        eval {
            require PureMessage::_sys;
            my %kv = PureMessage::_sys::lic_kv();

            if (exists $kv{'SerialNo'}) {
                $serial_no = $kv{SerialNo};
            }
        };

        if ($serial_no =~ /^[Ss]/) {
            $license_type = 's';
        }
        elsif ($serial_no =~ /^[Tt]/) {
            $license_type = 't';
        }

        eval {
            my $loc = PureMessage::Resources::Location->new();
            $location_id = $loc->get_location_id;
            $location_ip = $loc->ip;
        };

        my $id_string = "$serial_no $location_id $location_ip";
        $system_id = md5_hex($id_string);
        $system_id =~ s/^(.{8})(.{4})(.{4})(.{4})(.{12})$/$1-$2-$3-$4-$5/;

    }

    sub system_id {
        return $system_id;
    }

    sub license_type {
        return $license_type;
    }

    sub add_product_specific_headers {
        my $cb = shift;
        $cb->("PureMessage-SerialNo", $serial_no);
    }
}


{
    my @interesting_package_information = ();
    sub __get_interesting_package_information {
        my $prefix = shift || $config{prefix};
        my @interesting_packages = qw(
            PureMessage-AntiSpam-Data   
            PureMessage-AntiSpam-Engine 
            PureMessage-AntiSpam-Utils  
            PureMessage-Sophos-Data   
            PureMessage-Sophos-Engine 
            PureMessage-Sophos-SAVI   
            PureMessage-Blocklist        
            PureMessage-Blocklist-Daemon 
            PureMessage-Blocklist-Data
        );

        for my $package (@interesting_packages) {
            my $header_name = "Version-$package";
            my $header_value = "not installed";

            my $install_info = ActiveState::Install::installed(
                pkg => $package,
                etc => "$prefix/etc"
            );

            if ($install_info) {
                $header_value = $install_info->{version};
            }

            push @interesting_package_information, [
                $header_name,
                $header_value
            ];
        }
    }

    sub interesting_package_information {
        unless (@interesting_package_information) {
            __get_interesting_package_information($_[0]);
        }

        return @interesting_package_information;
    }
}

sub __ipv4_is_excluded {
    my @octet = split /\./, $_[0];
    # local
    if ($octet[0] == 127) { return 1; }
    # Class A non-routable
    if ($octet[0] == 10) { return 1; }
    # Class B non-routable
    if ($octet[0] == 172 && $octet[1] >= 16 && $octet[1] <= 31) { return 1; }
    # Class C non-routable
    if ($octet[0] == 192 && $octet[1] == 168) { return 1; }
    return 0;
}


sub first_untrusted_relay {
    my $message_info = shift;
    if (exists $message_info->{'fur'}) {
        return $message_info->{'fur'}[0];
    }
    elsif (exists $message_info->{'r'}) {
        return $message_info->{'r'}[0];
    }
    return;
}

sub exclude_from_log {
    my $fur = shift;
    my $message_info = shift;
    
    if (!defined $fur || __ipv4_is_excluded($fur)) {
        return 1;
    }

    unless($message_info) {
        return 0;
    }

    if ($message_info->{i}[0]) {
        # ??? the i tag marks internal mail ???
        return 1;
    }

    if ($message_info->{'Test-ID'}[0]) { 
        # AFAIK Test-ID is only added by pmx-test
        return 1;
    }

    return 0;
}

{
    my $upload_destination = $config{"mlog-stats"}{"Policy"}{"destination"};
    unless ($upload_destination =~ /\/$/) {
        $upload_destination .= '/';
    }
    sub upload_destination {
        return $upload_destination;
    }
}

sub message_log_location {
    my $prefix = shift || $config{prefix};
    return "$prefix/var/log/message_log";
}

sub blocklist_log_location {
    my $prefix = shift || $config{prefix};
    return "$prefix/var/log/blocklist_log";
}

sub state_file_location {
    my $prefix = shift || $config{prefix};
    return "$prefix/var/reports/mlog-stats-state.conf";
}

sub policy_siv_cache_location {
    my $prefix = shift || $config{prefix};
    return  "$prefix/etc/policy.siv.cache";
}

1;
