package PureMessage::MessageLog::Watch;
use strict;
use warnings;
use PureMessage::Config qw(%config expand_config_filename);
use PureMessage::MessageLog qw(message_log split_line);
use File::ReadBackwards;
use Carp;
use ActiveState::Config;
use HTTP::Date;
use Data::Dumper;

use base 'Exporter';
our @EXPORT_OK = qw(log_scan);

sub log_scan {
    my %ops = @_;
    
    # XXX allow prefix to be configurable?
    my $log = message_log;
    my $logger = PureMessage::MessageLog::Watch::Logger->new(
	'ident' => 'mlog_watch'
    );
    unless  (-e $log) {
	$logger->log('INFO', "Skipping scan, '$log' does not exist");
	return 0;
    }

    my $c = ActiveState::Config->new;
    my $conf_file = $ops{'config'} || expand_config_filename('logwatch');
    $c->parse_file($conf_file);

   
    my %conf = $c->as_hash;
    my $start_time = $ops{'start_time'} || time;
    my $lc = 0;
    my $window    = $ops{'scan_window'} || $conf{'scan_window'} || 30;
    $window *= 60;
    my $max_lines = $ops{'max_lines'}   || $conf{'max_lines'}   || 10000;
    my $max_rcpts = $ops{'max_rcpts'}   || $conf{'max_rcpts'}   || 50;
    my $max_msg_size 
	= $ops{'max_size'} || $conf{'max_size'} || 10 * (1024 * 1024);
    my (%relays, %senders, %rcpts, %size);
   
    $logger->vvlog("max_size: $max_msg_size");
    $logger->vvlog("max_lines: $max_lines");
    $logger->vvlog("max_rcpts: $max_rcpts");
    $logger->vvlog("scan_window: $window");
     
    my $bw = File::ReadBackwards->new($log);
    unless ($bw) {
	$logger->log('WARNING', 
	    "Scanning skipped. Can't read the log file '$log': $!");
	return 0;
    }

    my ($oldest, $newest) = (0,0);
    while (my $line = $bw->readline) {

	# internal mail should not count
	next if $line =~ / i / || $line =~ / i$/;
	my ($ts, @row) = split_line($line);
	$oldest = $ts unless $oldest;
	$newest = $ts unless $newest;
	my %hrow = @row;

	# XXX Log a warning if there were too many lines in the window 
	# period that not all of them could be read.
	if ($lc++ >= $max_lines) {
	    $logger->log('WARNING', 
		'Too many messages in scan window. Increase window size');
	    last;
	}
	if (($start_time - str2time($ts)) > $window) {
	    $logger->vlog("Out of scan range after $lc messages");
	    last;
	}

	# save the oldest and newest timestamps for messages that are examined
	$oldest = $ts if str2time($ts) < str2time($oldest);
	$newest = $ts if str2time($ts) > str2time($newest);

	$relays{$hrow{'r'}}++ if $hrow{'r'};
	$senders{$hrow{'f'}}++ if $hrow{'f'};
	$rcpts{$hrow{'r'}}++ if $hrow{'r'} && _count_rcpts(@row) >= $max_rcpts;
	$size{$hrow{'f'}}++ 
	    if $hrow{'f'} && $hrow{'size'} && $hrow{'size'} >= $max_msg_size;
    }

    unless ($lc > 0) {
	$logger->log('WARNING', "No messages were scanned");
    }

    # Check each counter against their high water threasholds
    # report on prolific relays
    my $hw_relay = $ops{'hw_relay'} || $conf{'hw_relay'} || 5000;
    $logger->vvlog("hw_relay: $hw_relay");
    foreach my $relay (keys %relays) {
    report(
	'relay'     => $relay,
	'counter'   => 'max_relay',
	'start'     => $newest,
	'end'       => $oldest,
	'count'     => $relays{$relay},
	'high_mark' => $hw_relay,
	) if $relays{$relay} >= $hw_relay;
   }

    # report on prolific senders
    my $hw_senders = $ops{'hw_senders'} || $conf{'hw_senders'} || 5000;
    $logger->vvlog("hw_senders: $hw_senders");
    foreach my $sender (keys %senders) {
	report(
	'sender'     => $sender,
	'counter'   => 'max_senders',
	'start'     => $newest,
	'end'       => $oldest,
	'count'     => $senders{$sender},
	'high_mark' => $hw_senders,
	) if $senders{$sender} >= $hw_senders;
    }
    
    # report on relays that send using lots of rcpts
    my $hw_rcpts = $ops{'hw_rcpts'} || $conf{'hw_rcpts'} || 5000;
    $logger->vvlog("hw_rcpts: $hw_rcpts");
    foreach my $relay (keys %rcpts) {
	report(
	'relay'     => $relay,
	'counter'   => 'max_rcpts',
	'start'     => $newest,
	'end'       => $oldest,
	'count'     => $rcpts{$relay},
	'high_mark' => $hw_rcpts,
	) if $rcpts{$relay} >= $hw_rcpts;
    }
   
    # report on senders who send big messages
    my $hw_size = $ops{'hw_size'} || $conf{'hw_size'} || 5000;
    $logger->vvlog("hw_size: $hw_size");
    foreach my $sender (keys %size) {
	report(
	'sender'    => $sender,
	'counter'   => 'max_size',
	'start'     => $newest,
	'end'       => $oldest,
	'count'     => $size{$sender},
	'high_mark' => $hw_size,
	) if $size{$sender} >= $hw_size;
    }
    
}

sub report {
    my %ops = @_;
    foreach (keys %ops) {
	print "$_: $ops{$_}\n" if $ops{$_};
    }
    print "\n";
}

sub _count_rcpts {
    my $i = 0;
    while(@_) {
	$i++ if shift(@_) eq 't';
	shift; # skip value
    }
    return $i;
}

package PureMessage::MessageLog::Watch::Logger;
use strict;
use warnings;
use base 'PureMessage::Logger';
use PureMessage::Config qw(%config);

sub vlog {
    my $self = shift;
    $self->log('DEBUG', shift) if $config{'debug_level'} >= 3;    
}

sub vvlog {
    my $self = shift;
    $self->log('DEBUG', shift) if $config{'debug_level'} >= 9;    
}

sub vvvvlog {
    my $self = shift;
    $self->log('DEBUG', shift) if $config{'debug_level'} >= 12;    
}
1;

